 /*
 * menu.c
 * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
 * 
 * ob-root-menu is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ob-root-menu is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * See the COPYING file. * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <locale.h>
#include <unistd.h>
#include <sys/stat.h>
#include <ctype.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>

#include <glib.h>
#include <glib/gi18n.h>

#include <menu-cache.h>

// Translations:
#include <libintl.h>
#include <locale.h>

int i, numDirs;

void print_separator();
void print_exec(const gchar*, const gchar*);
gchar* filter_cmd (const gchar*);
short file_exists(const char*);

int main (int argc , char **argv)
{
	MenuCacheDir *root;
	MenuCache *menu_cache = NULL;
	
	/* Setting the i18n environment */
	setlocale (LC_ALL, "");
	
	/* Localization */
	textdomain("ob-root-menu");
	bindtextdomain("ob-root-menu", "/usr/share/locale");
	bind_textdomain_codeset ("ob-root-menu", "UTF-8");
	
	printf("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n\t<openbox_pipe_menu xmlns=\"http://openbox.org/\"  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"  xsi:schemaLocation=\"http://openbox.org/\" >\n");

	print_exec(_("Execute"), "gmrun");
	print_separator();
	print_exec(_("Terminal"), "sakura");
	print_exec(_("Web Browser"), "x-www-browser");
	print_exec(_("File Manager"), "pcmanfm");
	if(argv[0]==1) print_exec(_("File Manager (root)"), "gtk-askpass pcmanfm");
	print_exec(_("Text Editor"), "geany");
	if(argv[0]==1) print_exec(_("Package Manager"), "gtk-askpass synaptic");
	if(!file_exists("/usr/bin/icecat")) {
		printf("<menu id=\"icecat-menu\" label=\"IceCat GNUzilla\">\n");
		print_exec(_("IceCat"), "icecat");
		print_exec(_("IceCat (New window)"), "icecat-bin -new-window");
		print_exec(_("IceCat (Private window)"), "icecat -private-window");
		printf("</menu>\n");		
	}
	print_separator();
	
	menu_cache = menu_cache_lookup_sync ("openbox-applications.menu");
	if (!menu_cache )
	{
		perror ("\nCannot connect to menu-cache\n");
		exit(-1);
	}	
	
	// BACKEND's root_dir (the first item in the hierarchy is "Aplications")
	root = menu_cache_dup_root_dir (menu_cache);
	
	// BACKEND's list ("Accesories", "Education", etc... and separators)
	GSList *children = menu_cache_dir_get_children (root);
	
	// Adding the dinamic part, only the parent items of the GSList
	// The children will be added in a separate thread,
	// improving the response of the popupmenu
	for ( ; children; children=children->next) 		
		switch ((gint) menu_cache_item_get_type (MENU_CACHE_ITEM(children->data)))
		{
			case MENU_CACHE_TYPE_DIR:
				{
					const gchar *name = menu_cache_item_get_name (MENU_CACHE_ITEM(children->data));
					
					if( !strcmp(name, "Sound & Video") )
						 printf("<menu id=\"openbox-Sound and Video\" label=\"Sound and Video\">\n");
					else printf("<menu id=\"openbox-%s\" label=\"%s\">\n", name, name);
					
					GSList *l = menu_cache_dir_get_children (MENU_CACHE_DIR(children->data));
					
					for ( ; l; l=l->next) 
						switch ((gint) menu_cache_item_get_type (MENU_CACHE_ITEM(l->data)))
						{
							case MENU_CACHE_TYPE_APP:
								{
									const gchar *name = menu_cache_item_get_name (MENU_CACHE_ITEM(l->data));
									const gchar *exec = menu_cache_app_get_exec (MENU_CACHE_APP(l->data));
									
									// Synaptic replicates:
									if( !strcmp("synaptic", exec) ) break;									
									
									if( !strcmp("gmrun", exec) ) break;
									if( !strcmp("pcmanfm --desktop-pref", exec) ) break;
									if( !strcmp("compton", exec) ) break;
									
									const gchar *cmd = filter_cmd(exec);
									
									printf("\t<item label=\"%s\">\n", name);
									printf("\t\t<action name=\"Execute\">\n");
									if(argv[0]==1) {
										if( !strcmp("synaptic-pkexec", cmd) ) 
											printf("\t\t\t<command><![CDATA[gtk-askpass synaptic]]></command>\n");
										else if( !strcmp("gparted-pkexec", cmd) ) 
											printf("\t\t\t<command><![CDATA[gtk-askpass gparted]]></command>\n");
										else printf("\t\t\t<command><![CDATA[%s]]></command>\n", cmd);
									}
									
									printf("\t\t</action>\n");
									printf("\t</item>\n");
								}
								break;
								
							default:
								break;
						}
						
					printf("</menu>\n");
				
				}
				break;

			case MENU_CACHE_TYPE_SEP:
				print_separator();
				break;
		}
		
	print_separator();
		
	printf("<menu id=\"configuration-menu\" label=\"%s\">\n", _("Configuration"));
		print_exec(_("Hot corners"), "geany ~/.config/hotcorners/hotcornersrc");
		printf("<menu id=\"obConfig-menu\" label=\"%s\">\n", _("OpenBox"));
			print_exec(_("Edit autostart"), "geany ~/.config/openbox/autostart");
			print_exec(_("Edit menu.xml"), "geany ~/.config/openbox/menu.xml");
			print_exec(_("Edit rc.xml"), "geany ~/.config/openbox/rc.xml");
			print_separator();
			print_exec(_("restart"), "openbox --restart");
			print_exec(_("reconfigure"), "openbox --reconfigure");
			print_exec(_("exit"), "openbox --exit");
		printf("</menu>\n");		
		printf("<menu id=\"touchpad-menu\" label=\"%s\">\n", _("Touchpad"));
			print_exec(_("On"), "synclient TouchPadOff=0");
			print_exec(_("Off"), "synclient TouchPadOff=1");
		printf("</menu>\n");
	printf("</menu>\n");
	printf("<menu id=\"screenshot-menu\" label=\"%s\">\n", _("Screenshot"));
		print_exec(_("Now"), "scrot '%Y-%m-%d--%s_$wx$h_scrot.png' -e 'mv $f ~/ &amp; viewnior ~/images/$f'");
		print_exec(_("In 5 seconds ..."), "scrot -d 5 '%Y-%m-%d--%s_$wx$h_scrot.png' -e 'mv $f ~/ &amp; viewnior ~/images/$f'");
		print_exec(_("In 10 seconds ..."), "scrot -d 10 '%Y-%m-%d--%s_$wx$h_scrot.png' -e 'mv $f ~/ &amp; viewnior ~/images/$f'");
		print_exec(_("Select an area ..."), "scrot -s '%Y-%m-%d--%s_$wx$h_scrot.png' -e 'mv $f ~/ &amp; viewnior ~/images/$f'");
	printf("</menu>\n");		
		
	print_separator();
	
	print_exec(_("Lock screen"), "lock-screen");
	print_exec(_("Exit"), "/usr/local/bin/shutdown");
	
		
	printf("</openbox_pipe_menu>\n");
 
	return EXIT_SUCCESS;
}

void print_separator()
{
	printf("<separator/>\n");
}

void print_exec(const gchar *name, const gchar *cmd)
{
	printf("<item label=\"%s\">\n", name);
	printf("\t<action name=\"Execute\">\n");
	printf("\t\t<command><![CDATA[%s]]></command>\n", cmd);
	printf("\t</action>\n");
	printf("</item>\n");	
}

gchar* filter_cmd (const gchar *exec)
{	
	g_return_val_if_fail(exec,"");

	GString *cmd = g_string_sized_new (64);
			
	for (;*exec; ++exec)
	{
		if (*exec == '%')
		{
			++exec;
			switch (*exec)
			{
				case 'u':
				case 'U': 
				case 'f':
				case 'F':
				case 'd':
				case 'D':
				case 'm':
				case 'n':
				case 'N':
				case 'v': 
					break;
					
				default:
					g_string_append_c (cmd, '%');
					g_string_append_c (cmd, *exec);
					break;
			}
		}
		else
			g_string_append_c (cmd, *exec);
	}
	
	return g_strchomp (g_string_free (cmd, FALSE));
}

short file_exists(const char *filename)
{
  int fd=open(filename, O_RDONLY);
  if (fd<0) return (errno==ENOENT)?-1:-2;    
  close(fd);
  return 0;
}
